@@ -5,6 +5,7 @@ extern crate rustc_interface;
 use clap::clap_app;
 use std::{
   env,
+  path::PathBuf,
   process::{exit, Command},
 };
 
@@ -16,7 +17,7 @@ fn main() {
 
   // FIXME(wcrichto): can we read the version directly from Cargo.toml?
   let matches = clap_app!(flowistry =>
-    (version: "0.3.8")
+    (version: "0.3.9")
     (author: "Will Crichton <wcrichto@cs.stanford.edu>")
     (@setting TrailingVarArg)
     (@subcommand rustc_version =>)
@@ -40,21 +41,27 @@ fn main() {
   )
   .get_matches_from(env::args().skip(1));
 
-  let mut args = match matches.subcommand() {
+  let (mut args, file_name) = match matches.subcommand() {
     ("rustc_version", _) => {
       let commit_hash = rustc_interface::util::commit_hash_str().unwrap_or("unknown");
       println!("{}", commit_hash);
       exit(0);
     }
-    ("backward_slice" | "forward_slice", Some(sub_m)) => vec![
-      ("FILE", sub_m.value_of("file").unwrap()),
-      ("START", sub_m.value_of("start").unwrap()),
-      ("END", sub_m.value_of("end").unwrap()),
-    ],
-    ("effects", Some(sub_m)) => vec![
-      ("FILE", sub_m.value_of("file").unwrap()),
-      ("POS", sub_m.value_of("pos").unwrap()),
-    ],
+    ("backward_slice" | "forward_slice", Some(sub_m)) => (
+      vec![
+        ("FILE", sub_m.value_of("file").unwrap()),
+        ("START", sub_m.value_of("start").unwrap()),
+        ("END", sub_m.value_of("end").unwrap()),
+      ],
+      sub_m.value_of("file").unwrap(),
+    ),
+    ("effects", Some(sub_m)) => (
+      vec![
+        ("FILE", sub_m.value_of("file").unwrap()),
+        ("POS", sub_m.value_of("pos").unwrap()),
+      ],
+      sub_m.value_of("file").unwrap(),
+    ),
     _ => {
       unimplemented!()
     }
@@ -66,14 +73,33 @@ fn main() {
   };
   args.push(("COMMAND", cmd));
 
+  let mut file_path = PathBuf::from(file_name);
+  let mut package = None;
+  while let Some(parent) = file_path.parent() {
+    file_path = parent.to_path_buf();
+
+    let manifest_path = file_path.join("Cargo.toml");
+    if manifest_path.exists() {
+      let manifest = cargo_toml::Manifest::from_path(manifest_path).unwrap();
+      package = Some(manifest.package.unwrap().name);
+      break;
+    }
+  }
+
   let mut cmd = Command::new(cargo_path);
   cmd
     .arg("check")
     .arg("-q")
-    .arg("--all")
     .args(flags)
     .env("RUSTC_WORKSPACE_WRAPPER", flowistry_rustc_path);
 
+  match package {
+    Some(package) => {
+      cmd.arg("-p").arg(package);
+    }
+    None => panic!("Could not find package for file: {}", file_path.display()),
+  };
+
   for (k, v) in args {
     cmd.env(format!("FLOWISTRY_{}", k), v);
   }
