@@ -9,7 +9,7 @@ use rustc_hir::{def_id::DefId, BodyId};
 use rustc_middle::{
   mir::{
     pretty::write_mir_fn,
-    visit::{NonUseContext, PlaceContext, Visitor},
+    visit::{MutatingUseContext, NonMutatingUseContext, NonUseContext, PlaceContext, Visitor},
     *,
   },
   ty::{
@@ -405,55 +405,81 @@ pub fn pointer_for_place(place: Place<'tcx>, tcx: TyCtxt<'tcx>) -> Option<Place<
     })
 }
 
-struct FindSpannedPlaces<'a, 'tcx> {
-  body: &'a Body<'tcx>,
-  span: Span,
-  places: HashSet<(Place<'tcx>, Location)>,
-}
+pub fn span_to_places(body: &Body<'tcx>, span: Span) -> (Vec<(Place<'tcx>, Location)>, Vec<Span>) {
+  struct FindSpannedPlaces<'a, 'tcx> {
+    body: &'a Body<'tcx>,
+    span: Span,
+    places: HashSet<(Place<'tcx>, Location)>,
+    place_spans: Vec<Span>,
+  }
 
-impl Visitor<'tcx> for FindSpannedPlaces<'_, 'tcx> {
-  fn visit_place(&mut self, place: &Place<'tcx>, context: PlaceContext, location: Location) {
-    // assignments to return value have a span of the entire function, which we want to avoid
-    if let Some(local) = place.as_local() {
-      if local.as_usize() == 0 {
-        return;
+  impl Visitor<'tcx> for FindSpannedPlaces<'_, 'tcx> {
+    fn visit_place(&mut self, place: &Place<'tcx>, context: PlaceContext, location: Location) {
+      // Assignments to return value have a span of the entire function, which we want to avoid
+      if let Some(local) = place.as_local() {
+        if local.as_usize() == 0 {
+          return;
+        }
       }
-    }
 
-    let span = match context {
-      PlaceContext::MutatingUse(..) | PlaceContext::NonMutatingUse(..) => {
-        let source_info = self.body.source_info(location);
-        Some(source_info.span)
-      }
-      PlaceContext::NonUse(non_use) => match non_use {
-        NonUseContext::VarDebugInfo => {
-          if self.body.args_iter().any(|local| local == place.local) {
-            let source_info = self.body.local_decls()[place.local].source_info;
-            Some(source_info.span)
-          } else {
-            None
-          }
+      // Three cases, shown by example:
+      //   fn foo(x: i32) {
+      //     let y = x + 1;
+      //   }
+      // If the user selects...
+      // * "x: i32" -- this span is contained in the LocalDecls for _1,
+      //   which is represented by NonUseContext::VarDebugInfo
+      // * "x + 1" -- MIR will generate a temporary to assign x into, whose
+      //   span is given to "x". That corresponds to MutatingUseContext::Store
+      // * "y" -- this corresponds to NonMutatingUseContext::Inspect
+      let span = match context {
+        PlaceContext::MutatingUse(MutatingUseContext::Store)
+        | PlaceContext::NonMutatingUse(NonMutatingUseContext::Inspect) => {
+          let source_info = self.body.source_info(location);
+          Some(source_info.span)
+        }
+        PlaceContext::NonUse(NonUseContext::VarDebugInfo)
+          if self.body.args_iter().any(|local| local == place.local) =>
+        {
+          let source_info = self.body.local_decls()[place.local].source_info;
+          Some(source_info.span)
         }
         _ => None,
-      },
-    };
+      };
 
-    if let Some(span) = span {
-      if self.span.contains(span) || span.contains(self.span) {
-        self.places.insert((*place, location));
+      if let Some(span) = span {
+        if self.span.contains(span) || span.contains(self.span) {
+          self.places.insert((*place, location));
+          self.place_spans.push(span);
+        }
       }
     }
   }
-}
 
-pub fn span_to_places(body: &Body<'tcx>, span: Span) -> Vec<(Place<'tcx>, Location)> {
   let mut visitor = FindSpannedPlaces {
     body,
     span,
     places: HashSet::default(),
+    place_spans: Vec::new(),
   };
   visitor.visit_body(body);
-  visitor.places.into_iter().collect::<Vec<_>>()
+
+  let places = visitor.places.into_iter().collect::<Vec<_>>();
+
+  // Find the smallest spans that describe the sliced places
+  let mut spans = Vec::new();
+  visitor
+    .place_spans
+    .sort_by_key(|span| span.hi() - span.lo());
+  for span in visitor.place_spans.into_iter() {
+    if spans.iter().any(|other| span.contains(*other)) {
+      continue;
+    }
+
+    spans.push(span);
+  }
+
+  (places, spans)
 }
 
 pub fn dump_results<'tcx, A>(
